@@ -2,27 +2,18 @@ import asyncio
 import logging
 import random
 import ssl
+from collections import defaultdict
 from ssl import SSLContext, PROTOCOL_TLS
 from typing import Optional, Union, Any
-from deprecation import deprecated
 from concurrent.futures import CancelledError
 from .buildin import Buildin
-from .protocol import Proto
-from .protocol import Protocol
-from .abc.events import Events
-from ..util.convert import convert
-from ..exceptions import ForbiddenError, NodeError, AuthError
-
-
-
-_WATCH_MISSING = \
-    'auto reconnect cannot act on node changes since `WATCH` privileges on ' \
-    'the `@node` scope are missing'
+from .protocol import Proto, Protocol
+from ..exceptions import NodeError, AuthError
 
 
 class Client(Buildin):
 
-    MAX_RECONNECT_WAIT_TIME = 120
+    MAX_RECONNECT_WAIT_TIME = 60
     MAX_RECONNECT_TIMEOUT = 10
 
     def __init__(
@@ -38,10 +29,9 @@ class Client(Buildin):
                 When set to `True`, the client will automatically
                 reconnect when a connection is lost. If set to `False` and the
                 connection gets lost, one may call the `reconnect()` method to
-                make a new connection. The auto-reconnect option can act on
-                node changes and does so automatically if the connected user
-                has the required `WATCH` privileges on the `@node` scope.
-                Defaults to `True`.
+                make a new connection. The auto-reconnect option will listen to
+                node changes and automatically start a reconnect loop if the
+                *shutting-down* status is received from the node.
             ssl (SSLContext or bool, optional):
                 Accepts an ssl.SSLContext for creating a secure connection
                 using SSL/TLS. This argument may simply be set to `True` in
@@ -63,34 +53,23 @@ class Client(Buildin):
         self._scope = '@t'  # default to thingsdb scope
         self._pool_idx = 0
         self._reconnecting = False
+        self._rooms = dict()
+        self._rooms_lock = asyncio.Lock()
+
         if ssl is True:
             self._ssl = SSLContext(PROTOCOL_TLS)
         elif ssl is False:
             self._ssl = None
         else:
             self._ssl = ssl
-        self._event_handlers = []
 
-    def add_event_handler(self, event_handler: Events) -> None:
-        """Add an event handler.
+    def get_rooms(self):
+        """Can be used to get the rooms which are joined.
 
-        Event handlers will called in the order they are added.
-
-        Args:
-            event_handler (Events):
-                An instance of Events (see thingsdb.client.abc.events).
+        Returns:
+            a tuple with unique Room instances.
         """
-        self._event_handlers.append(event_handler)
-
-    def remove_event_handler(self, event_handler: Events) -> None:
-        """Remove an event handler.
-
-        Args:
-            event_handler (Events):
-                An instance of Events (see thingsdb.client.abc.events).
-        """
-        self._event_handlers.remove(event_handler)
-
+        return tuple(self._rooms.values())
 
     def get_event_loop(self) -> asyncio.AbstractEventLoop:
         """Can be used to get the event loop.
@@ -297,18 +276,11 @@ class Client(Buildin):
         self._auth = self._auth_check(auth)
         await self._authenticate(timeout)
 
-        if self._reconnect:
-            try:
-                await self.watch(scope='@n')
-            except ForbiddenError:
-                logging.warning(_WATCH_MISSING)
-
     def query(
             self,
             code: str,
             scope: Optional[str] = None,
             timeout: Optional[int] = None,
-            convert_vars: bool = True,
             **kwargs: Any
     ) -> asyncio.Future:
         """Query ThingsDB.
@@ -326,13 +298,6 @@ class Client(Buildin):
                 Raise a time-out exception if no response is received within X
                 seconds. If no time-out is given, the client will wait forever.
                 Defaults to `None`.
-            convert_vars (bool, optional):
-                Only applicable if `**kwargs` are given. If set to `True`, then
-                the provided **kwargs values will be converted so ThingsDB can
-                understand them. For example, a thing should be given just by
-                it's ID and with conversion the `#` will be extracted. When
-                this argument is `False`, the **kwargs stay untouched.
-                Defaults to `True`.
             **kwargs (any, optional):
                 Can be used to inject variable into the ThingsDB code.
 
@@ -360,13 +325,9 @@ class Client(Buildin):
             scope = self._scope
 
         code = code.strip()  # strip white space characters
-
+        data = [scope, code]
         if kwargs:
-            if convert_vars:
-                kwargs = {k: convert(v) for k, v in kwargs.items()}
-            data = [scope, code, kwargs]
-        else:
-            data = [scope, code]
+            data.append(kwargs)
 
         return self._write_pkg(Proto.REQ_QUERY, data, timeout=timeout)
 
@@ -379,7 +340,7 @@ class Client(Buildin):
     ) -> asyncio.Future:
         while True:
             if not self.is_connected():
-                logging.info('wait for a connection')
+                logging.info('Wait for a connection')
                 await asyncio.sleep(1.0)
                 continue
 
@@ -387,7 +348,7 @@ class Client(Buildin):
                 res = await self._protocol.write(tp, data, is_bin, timeout)
             except (CancelledError, NodeError, AuthError) as e:
                 logging.error(
-                    f'error sending package: '
+                    f'Failed to transmit package: '
                     f'{e}({e.__class__.__name__}) (will try again)')
                 await asyncio.sleep(1.0)
                 continue
@@ -411,7 +372,6 @@ class Client(Buildin):
             *args: Optional[Any],
             scope: Optional[str] = None,
             timeout: Optional[int] = None,
-            convert_args: bool = True,
             **kwargs: Any,
     ) -> asyncio.Future:
         """Run a procedure.
@@ -434,13 +394,6 @@ class Client(Buildin):
                 Raise a time-out exception if no response is received within X
                 seconds. If no time-out is given, the client will wait forever.
                 Defaults to `None`.
-            convert_args (bool, optional):
-                Only applicable if `*args` are given. If set to `True`, then
-                the provided `*args` values will be converted so ThingsDB can
-                understand them. For example, a thing should be given just by
-                it's ID and with conversion the `#` will be extracted. When
-                this argument is `False`, the `*args` stay untouched.
-                Defaults to `True`.
             **kwargs (any):
                 Arguments which are injected as the procedure arguments.
                 Instead of by name, the arguments may also be parsed using
@@ -460,60 +413,86 @@ class Client(Buildin):
         if scope is None:
             scope = self._scope
 
-        if kwargs:
-            args = {
-                k: convert(v)
-                for k, v in kwargs.items()
-            } if convert_args else kwargs
-        elif args and convert_args:
-            args = [convert(arg) for arg in args]
+        data = [scope, procedure]
 
-        return self._write_pkg(
-            Proto.REQ_RUN,
-            [scope, procedure, args],
-            timeout=timeout)
+        if args:
+            data.append(args)
+            if kwargs:
+                raise ValueError(
+                    'it is not possible to use both keyword arguments '
+                    'and positional arguments at the same time')
+        elif kwargs:
+            data.append(kwargs)
 
-    def watch(self, *ids: int, scope: Optional[str] = None) -> asyncio.Future:
-        """Subscribe for changes on given things.
+        return self._write_pkg(Proto.REQ_RUN, data, timeout=timeout)
 
-        This method accepts one or more thing ids to subscribe to. This
-        method will simply return None as soon as the subscribe request is
-        successful handled by ThingsDB. After the response, the client will
-        receive `INIT` events for all subscribed ids. After that, ThingsDB
-        will continue to provide the client with `UPDATE` events which contain
-        changes to the subscribed thing. A `DELETE` event might be received
-        if, and only if the thing is removed and garbage collected from the
-        collection.
+    def _emit(
+            self,
+            room_id: int,
+            event: str,
+            *args: Optional[Any],
+            scope: Optional[str] = None,
+    ) -> asyncio.Future:
+        """Emit an event.
+
+        Use Room(room_id, scope=scope).emit(..) instead of this function to
+        emit an event to a roomId.
 
         Args:
-            *ids (int):
-                Thing IDs to subscribe to. No error is returned in case one of
-                the given things are not found within the collection, instead a
-                `WARN` event will be send to the client.
+            room_id (int):
+                Room Id to emit the event to.
+            event (str):
+                Name of the event to emit.
+            *args:
+                Additional argument to send with the event.
             scope (str, optional):
-                Subscribe on things in this scope. If not specified, the
+                Find the room in this scope. If not specified, the
                 default scope will be used. Only collection scopes may contain
-                things so only collection scopes can be used.
+                rooms so only collection scopes can be used.
                 See https://docs.thingsdb.net/v0/overview/scopes/ for how to
                 format a scope.
 
         Returns:
             asyncio.Future (None):
-                Future which result will be set to `None` if successful.
+                Future which should be awaited. The result of the future will
+                be set to `None` when successful.
+        """
+        if scope is None:
+            scope = self._scope
+        return self._write_pkg(Proto.REQ_EMIT, [scope, room_id, event, *args])
+
+    def _join(self, *ids: int, scope: Optional[str] = None) -> asyncio.Future:
+        """Join one or more rooms.
+
+        Args:
+            *ids (int):
+                Room Ids to join. No error is returned in case one of
+                the given room Ids are not found within the collection.
+                Instead, the return value will contain `None` instead of the
+                Id in the returned list.
+            scope (str, optional):
+                Join room(s) in this scope. If not specified, the
+                default scope will be used. Only collection scopes may contain
+                rooms so only collection scopes can be used.
+                See https://docs.thingsdb.net/v0/overview/scopes/ for how to
+                format a scope.
+
+        Returns:
+            asyncio.Future ([*ids]):
+                Returns a Future which result will be set to a `list` with all
+                the room Ids from the request. If, and only if a given room Id
+                was not found in the collection, then the room Id at this
+                position in the list will be `None`.
         """
         if scope is None:
             scope = self._scope
 
-        return self._write_pkg(Proto.REQ_WATCH, [scope, *ids])
+        return self._write_pkg(Proto.REQ_JOIN, [scope, *ids])
 
-    def unwatch(
-            self,
-            *ids: int,
-            scope: Optional[str] = None
-    ) -> asyncio.Future:
-        """Unsubscribe for changes on given things.
+    def _leave(self, *ids: int, scope: Optional[str] = None) -> asyncio.Future:
+        """Leave one or more rooms.
 
-        Stop receiving events for the things given by one or more ids. It is
+        Stop receiving events for the rooms given by one or more ids. It is
         possible that the client receives an event shortly after calling the
         unsubscribe method because the event was queued.
 
@@ -530,13 +509,16 @@ class Client(Buildin):
                 format a scope.
 
         Returns:
-            asyncio.Future (None):
-                Future which result will be set to `None` if successful.
+            asyncio.Future ([*ids]]):
+                Returns a Future which result will be set to a `list` with all
+                the room Ids from the request. If, and only if a given room Id
+                was not found in the collection, then the room Id at this
+                position in the list will be `None`.
         """
         if scope is None:
             scope = self._scope
 
-        return self._write_pkg(Proto.REQ_UNWATCH, [scope, *ids])
+        return self._write_pkg(Proto.REQ_LEAVE, [scope, *ids])
 
     @staticmethod
     def _auth_check(auth):
@@ -571,20 +553,45 @@ class Client(Buildin):
             self._pool_idx += 1
             self._pool_idx %= len(self._pool)
 
-    def _on_event(self, pkg):
-        if pkg.tp == Proto.ON_NODE_STATUS and \
-                self._reconnect and \
-                pkg.data == 'SHUTTING_DOWN':
-            asyncio.ensure_future(self.reconnect(), loop=self._loop)
+    async def _on_room(self, room_id, pkg):
+        async with self._rooms_lock:
+            try:
+                room = self._rooms[room_id]
+            except KeyError:
+                logging.warn(
+                    f'Got an event (tp:{pkg.tp}) for room Id {room_id} but '
+                    f'the room is not known by the ThingsDB client')
+            else:
+                room._on_event(pkg)
 
-        for event_handler in self._event_handlers:
-            event_handler(pkg.tp, pkg.data)
+    def _on_event(self, pkg):
+        if pkg.tp == Proto.ON_NODE_STATUS:
+            status, node_id = pkg.data['status'], pkg.data['id']
+
+            if self._reconnect and status == 'SHUTTING_DOWN':
+                asyncio.ensure_future(self.reconnect(), loop=self._loop)
+
+            logging.debug(
+                f'Node with Id {node_id} has changed its status to: {status}')
+            return
+
+        try:
+            room_id = pkg.data['id']
+        except KeyError:
+            if pkg.tp == Proto.ON_WARN:
+                warn = pkg.data
+                logging.warn(
+                    f'Warning from ThingsDB: '
+                    f'{warn["warn_msg"]} ({warn["warn_code"]})')
+            else:
+                logging.warn(f'Unexpected event: tp:{pkg.tp} data:{pkg.data}')
+        else:
+            asyncio.ensure_future(self._on_room(room_id, pkg), loop=self._loop)
 
     def _on_connection_lost(self, protocol, exc):
         if self._protocol is not protocol:
             return
         self._protocol = None
-
         if self._reconnect:
             asyncio.ensure_future(self.reconnect(), loop=self._loop)
 
@@ -598,11 +605,12 @@ class Client(Buildin):
                 await self._connect(timeout=timeout)
                 await self._ping(timeout=2)
                 await self._authenticate(timeout=5)
+                await self._rejoin()
             except Exception as e:
                 logging.error(
-                    f'connecting to {host}:{port} failed: '
+                    f'Connecting to {host}:{port} failed: '
                     f'{e}({e.__class__.__name__}), '
-                    f'try next connect in {wait_time} seconds'
+                    f'Try next connect in {wait_time} seconds'
                 )
             else:
                 if protocol and protocol.transport:
@@ -615,27 +623,23 @@ class Client(Buildin):
             wait_time = min(wait_time, self.MAX_RECONNECT_WAIT_TIME)
             timeout = min(timeout+1, self.MAX_RECONNECT_TIMEOUT)
 
-        if self._reconnect:
-            try:
-                await self.watch(scope='@n')
-            except ForbiddenError:
-                logging.warning(_WATCH_MISSING)
-
-        for event_handler in self._event_handlers:
-            event_handler.on_reconnect()
-
     def _ping(self, timeout):
         return self._write(Proto.REQ_PING, timeout=timeout)
 
     def _authenticate(self, timeout):
         return self._write(Proto.REQ_AUTH, data=self._auth, timeout=timeout)
 
-    @deprecated(details='Use `set_default_scope` instead')
-    def use(self, scope):
-        if not scope.startswith('@'):
-            scope = f'@{scope}' if scope.startswith(':') else f'@:{scope}'
-        self.set_default_scope(scope)
+    async def _rejoin(self):
+        if not self._rooms:
+            return  # do nothig if no rooms are used
 
-    @deprecated(details='Use `get_default_scope` instead')
-    def get_scope(self):
-        return self._scope
+        # re-arrange the rooms per scope to combine joins in a less requests
+        scopes = defaultdict(list)
+        for room in self._rooms.values():
+            if room.id:
+                scopes[room.scope].append(room.id)
+
+        # join request per scope, each for one or more rooms
+        await asyncio.gather(*[
+            self._join(*ids, scope=scope)
+            for scope, ids in scopes.items()])
